#!/bin/bash

powerdevilrc_path="$HOME/.kde/share/config/powerdevilrc"
profilerc_path="$HOME/.kde/share/config/powerdevilprofilesrc"

del_temp()
{
	echo -e "\n\n\033[1m使用者已中斷操作（Ctrl-C）\033[0m\n" 1>&2
	rm -f /tmp/powerdevilrc.$$
	rm -f /tmp/powerdevilprofilesrc.$$
	exit 1
}

gconf_failure_end()
{
	echo -e "\n\033[1m設定失敗：gconftool-2 執行錯誤\033[0m\n" 1>&2
}

write_failure_end()
{
	echo -e "\n\033[1m設定失敗：Powerdevil 設定檔寫入錯誤\033[0m\n" 1>&2
	rm -f /tmp/powerdevilrc.$$
	rm -f /tmp/powerdevilprofilesrc.$$
}

GNOME_setting()
{
	gconftool-2 --type int --set /apps/gnome-screensaver/idle_delay 0 || ( gconf_failure_end; return 1 )
	gconftool-2 --type string --set /apps/gnome-power-manager/actions/sleep_type_ac "hibernate" || ( gconf_failure_end; return 1 )
	gconftool-2 --type string --set /apps/gnome-power-manager/actions/sleep_type_battery "hibernate" || ( gconf_failure_end; return 1 )
	
	gconftool-2 --type int --set /apps/gnome-power-manager/timeout/sleep_display_ac $1 || ( gconf_failure_end; return 1 )
	gconftool-2 --type int --set /apps/gnome-power-manager/timeout/sleep_computer_ac $2 || ( gconf_failure_end; return 1 )
	gconftool-2 --type int --set /apps/gnome-power-manager/timeout/sleep_display_battery $1 || ( gconf_failure_end; return 1 )
	gconftool-2 --type int --set /apps/gnome-power-manager/timeout/sleep_computer_battery $2 || ( gconf_failure_end; return 1 )
	
	return 0
}

KDE_setting()
{
	BAKIFS=$IFS
	IFS=$( echo -e -n "\n\b" )
	
	# Read file powerdevilrc
	
	powerdevilrc=( $( cat $powerdevilrc_path ) )
	
	IFS=$BAKIFS
	
	# Whether to modify powerdevilrc or not
	
	rewrite=0
	ACProfileKnown=0
	BatteryProfileKnown=0
	ProfilesIndex=0
	line_no=${#powerdevilrc[*]}
	
	for (( i = 0; i < line_no; ++i )); do
		if [[ ${powerdevilrc[$i]} =~ \[Profiles\] ]]; then
			ProfilesIndex=$i
		elif [[ ${powerdevilrc[$i]} =~ manageDPMS=(.*) ]]; then
			rewrite=1
			powerdevilrc[$i]="manageDPMS=true"
		elif [[ ${powerdevilrc[$i]} =~ ACProfile=(.*) ]]; then
			ACProfileKnown=1
			ac_profile=${BASH_REMATCH[1]}
			if [ -z "$ac_profile" ]; then
				rewrite=1
				ac_profile="EPA Powersave"
				powerdevilrc[$i]="ACProfile=EPA Powersave"
			fi
		elif [[ ${powerdevilrc[$i]} =~ BatteryProfile=(.*) ]]; then
			BatteryProfileKnown=1
			battery_profile=${BASH_REMATCH[1]}
			if [ -z "$battery_profile" ]; then
				rewrite=1
				battery_profile="EPA Powersave"
				powerdevilrc[$i]="BatteryProfile=EPA Powersave"
			fi
		fi
	done
	
	BAKIFS=$IFS
	IFS=""
	
	# Add missing [Profiles] if needed
	
	if [ $ProfilesIndex -eq 0 ]; then
		ACProfileKnown=0
		BatteryProfileKnown=0
		ProfilesIndex=${#powerdevilrc[*]}
		powerdevilrc[$ProfilesIndex]="[Profiles]"
	fi
	
	# Add missing ACProfile / BatteryProfile
	
	if [ $ACProfileKnown -eq 0 ]; then
		rewrite=1
		ac_profile="EPA Powersave"
		powerdevilrc[$ProfilesIndex]=$( echo -e "${powerdevilrc[$ProfilesIndex]}\nACProfile=EPA Powersave" )
	fi
	
	if [ $BatteryProfileKnown -eq 0 ]; then
		rewrite=1
		battery_profile="EPA Powersave"
		powerdevilrc[$ProfilesIndex]=$( echo -e "${powerdevilrc[$ProfilesIndex]}\nBatteryProfile=EPA Powersave" )
	fi
	
	IFS=$BAKIFS
	
	# Rewrite powerdevilrc if needed
	
	if [ $rewrite -eq 1 ]; then
		if [ -n "${powerdevilrc[0]}" ]; then
			echo "${powerdevilrc[0]}" > /tmp/powerdevilrc.$$
		fi
		
		line_no=${#powerdevilrc[*]}
		
		for (( i = 1; i < line_no; ++i )); do
			if [[ ${powerdevilrc[$i]} =~ \[(.+)\] ]]; then
				echo "" >> /tmp/powerdevilrc.$$
			fi
			echo "${powerdevilrc[$i]}" >> /tmp/powerdevilrc.$$
		done
		
		rm -f $powerdevilrc_path || ( write_failure_end; return 1 )
		mv /tmp/powerdevilrc.$$ $powerdevilrc_path || ( write_failure_end; return 1 )
	fi
	
	# ACProfile and BatteryProfile are found
	
	BAKIFS=$IFS
	IFS=$( echo -e -n "\n\b" )
	
	# Read file powerdevilprofilesrc
	
	profilerc=( $( cat $profilerc_path ) )
	
	IFS=$BAKIFS
	line_no=${#profilerc[*]}
	
	# Change ACProfile and BatteryProfile settings
	
	SetACProfile=0
	SetBatteryProfile=0
	
	for (( i = 0; i < line_no; ++i )); do
		if [[ ${profilerc[$i]} =~ \[(.+)\] ]]; then
			if [ "${BASH_REMATCH[1]}" == "$ac_profile" -o "${BASH_REMATCH[1]}" == "$battery_profile" ]; then
				if [ "${BASH_REMATCH[1]}" == "$ac_profile" ]; then
					SetACProfile=1
				fi
				
				if [ "${BASH_REMATCH[1]}" == "$battery_profile" ]; then
					SetBatteryProfile=1
				fi
				
				# Modify existing settings
				
				ProfileIndex=$i
				SetDimOnIdle=0
				SetDimOnIdleTime=0
				SetIdleAction=0
				SetIdleTime=0
				
				for (( i = i + 1; i < line_no; ++i )); do
					if [[ ${profilerc[$i]} =~ dimOnIdle= ]]; then
						if [ $1 -gt 0 ]; then
							profilerc[$i]="dimOnIdle=true"
						else
							profilerc[$i]="dimOnIdle=false"
						fi
						SetDimOnIdle=1
					elif [[ ${profilerc[$i]} =~ dimOnIdleTime= ]]; then
						if [ $1 -gt 0 ]; then
							profilerc[$i]="dimOnIdleTime=$1"
							SetDimOnIdleTime=1
						fi
					elif [[ ${profilerc[$i]} =~ idleAction= ]]; then
						if [ $2 -gt 0 ]; then
							profilerc[$i]="idleAction=4"
						else
							profilerc[$i]="idleAction=1"
						fi
						SetIdleAction=1
					elif [[ ${profilerc[$i]} =~ idleTime= ]]; then
						if [ $2 -gt 0 ]; then
							profilerc[$i]="idleTime=$2"
							SetIdleTime=1
						fi
					elif [[ ${profilerc[$i]} =~ \[.+\] ]]; then
						let --i
						break
					fi
				done
				
				# Add missing settings if needed
				
				BAKIFS=$IFS
				IFS=""
				
				if [ $SetDimOnIdle -eq 0 ]; then
					if [ $1 -gt 0 ]; then
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\ndimOnIdle=true" )
					else
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\ndimOnIdle=false" )
					fi
				fi
				
				if [ $SetDimOnIdleTime -eq 0 ]; then
					if [ $1 -gt 0 ]; then
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\ndimOnIdleTime=$1" )
					fi
				fi
				
				if [ $SetIdleAction -eq 0 ]; then
					if [ $1 -gt 0 ]; then
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\nidleAction=4" )
					else
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\nidleAction=1" )
					fi
				fi
				
				if [ $SetIdleTime -eq 0 ]; then
					if [ $1 -gt 0 ]; then
						profilerc[$ProfileIndex]=$( echo -e -n "${profilerc[$ProfileIndex]}\nidleTime=$2" )
					fi
				fi
				
				IFS=$BAKIFS
				
				# Missing settings added
			fi
		fi
	done
	
	# Write settings to a temp file
	
	if [ -n "${profilerc[0]}" ]; then
		echo "${profilerc[0]}" > /tmp/powerdevilprofilesrc.$$
	fi
	
	line_no=${#profilerc[*]}
	
	for (( i = 1; i < line_no; ++i )); do
		if [[ ${profilerc[$i]} =~ \[(.+)\] ]]; then
			echo "" >> /tmp/powerdevilprofilesrc.$$
		fi
		echo "${profilerc[$i]}" >> /tmp/powerdevilprofilesrc.$$
	done
	
	if [ $SetACProfile -eq 0 -o $SetBatteryProfile -eq 0 ]; then
		BAKIFS=$IFS
		IFS=""
		
		if [ $1 -gt 0 ]; then
			settings="dimOnIdle=true"
		else
			settings="dimOnIdle=false"
		fi
		
		if [ $1 -gt 0 ]; then
			settings=$( echo -e "${settings}\ndimOnIdleTime=$1" )
		fi
		
		if [ $2 -gt 0 ]; then
			settings=$( echo -e "${settings}\nidleAction=4" )
		else
			settings=$( echo -e "${settings}\nidleAction=1" )
		fi
		
		if [ $2 -gt 0 ]; then
			settings=$( echo -e "${settings}\nidleTime=$2" )
		fi
		
		IFS=$BAKIFS
		
		if [ $SetACProfile -eq 0 ]; then
			echo -e "\n[$ac_profile]" >> /tmp/powerdevilprofilesrc.$$
			echo -e "$settings" >> /tmp/powerdevilprofilesrc.$$
		fi
		
		if [ $SetBatteryProfile -eq 0 -a "$battery_profile" != "$ac_profile" ]; then
			echo -e "\n[$battery_profile]" >> /tmp/powerdevilprofilesrc.$$
			echo -e "$settings" >> /tmp/powerdevilprofilesrc.$$
		fi
	fi
	
	# Write file powerdevilprofilesrc
	
	rm $profilerc_path || ( write_failure_end; return 1 )
	mv /tmp/powerdevilprofilesrc.$$ $profilerc_path || ( write_failure_end; return 1 )
	
	return 0
}

# Handle Ctrl-C interrupt

trap del_temp 2

# Read preferred hibernate values

echo -e -n "\n\033[1m顯示器休眠需等候閒置時間（分鐘，0＝永不，Enter＝預設值）：\033[0m"
read sleep_display

if [ -n "$sleep_display" ]; then
	if [[ "$sleep_display" =~ ^[[:digit:]]+$ ]]; then
		if [ $sleep_display -eq 0 ]; then
			echo -e "使用自訂值\033[1m（永不）\033[0m\n"
		else
			echo -e "使用自訂值\033[1m（$sleep_display 分鐘）\033[0m\n"
		fi
	else
		sleep_display=10
		echo -e "使用預設值\033[1m（10 分鐘）\033[0m\n"
	fi
else
	sleep_display=10
	echo -e "使用預設值\033[1m（10 分鐘）\033[0m\n"
fi

echo -e -n "\033[1m電腦休眠需等候閒置時間（分鐘，0＝永不，Enter＝預設值）：\033[0m"
read sleep_computer

if [ -n "$sleep_computer" ]; then
	if [[ $sleep_computer =~ ^[[:digit:]]+$ ]]; then
		if [ $sleep_computer -eq 0 ]; then
			echo -e "使用自訂值\033[1m（永不）\033[0m\n"
		else
			echo -e "使用自訂值\033[1m（$sleep_computer 分鐘）\033[0m\n"
		fi
	else
		sleep_computer=30
		echo -e "使用預設值\033[1m（30 分鐘）\033[0m"
	fi
else
	sleep_computer=30
	echo -e "使用預設值\033[1m（30 分鐘）\033[0m"
fi

# Set KDE

if [ -f "$powerdevilrc_path" -a -f "$profilerc_path" ]; then
	echo -e "\n\033[1m找到 KDE Powerdevil 設定檔，進行設定...\033[0m\n"
	if KDE_setting $sleep_display $sleep_computer; then
		echo -e "\033[1mKDE 電源管理設定成功！\033[0m"
		if which kcmshell4 > /dev/null; then
			echo -e "\033[1m按下 Enter 鍵觀看設定結果...\033[0m"
			read
			kcmshell4 powerdevilconfig 1>/dev/null 2>/dev/null &
		fi
	fi
else
	echo -e "\n\033[1m略過 KDE 電源管理設定：找不到 Powerdevil 設定檔\033[0m\n" 1>&2
fi

# Set GNOME

if which gconftool-2 > /dev/null; then
	echo -e "\n\033[1m找到 GNOME gconftool-2，進行設定...\033[0m\n"
	if GNOME_setting `expr $sleep_display \* 60` `expr $sleep_computer \* 60`; then
		echo -e "\033[1mGNOME 電源管理設定成功！\033[0m"
		if which gnome-power-preferences > /dev/null; then
			echo -e "\033[1m按下 Enter 鍵觀看設定結果...\033[0m"
			read
			gnome-power-preferences 1>/dev/null 2>/dev/null &
		fi
	fi
else
	echo -e "\n\033[1m略過 GNOME 電源管理設定：無法執行 gconftool-2\033[0m\n" 1>&2
fi

